import { Switch, Route, useLocation } from 'wouter';
import { queryClient } from './lib/queryClient';
import { QueryClientProvider } from '@tanstack/react-query';
import { Toaster } from '@/components/ui/toaster';
import NotFound from '@/pages/not-found';
import Home from '@/pages/Home';
import PricingPage from '@/pages/PricingPage';
import Terms from '@/pages/Terms';
import Privacy from '@/pages/Privacy';
import { AnimatePresence } from 'framer-motion';
import { useEffect, useState, lazy, Suspense } from 'react';
import ErrorBoundary from '@/components/core/ErrorBoundary';
import { Helmet } from 'react-helmet-async';
import { startMeasure, endMeasure, observeCoreWebVitals } from './utils/performance';
import IntercomMessenger from '@/components/IntercomMessenger';

// Component to scroll to top on route change
function ScrollToTop() {
  const [location] = useLocation();

  useEffect(() => {
    window.scrollTo(0, 0);
  }, [location]);

  return null;
}

// Component for monitoring page change performance
function PageChangeMonitor() {
  const [location] = useLocation();
  
  useEffect(() => {
    // Start measuring page change performance
    startMeasure('page-change');
    
    // Schedule measurement for after render
    const timeoutId = setTimeout(() => {
      endMeasure('page-change');
    }, 0);
    
    return () => clearTimeout(timeoutId);
  }, [location]);
  
  return null;
}

function Router() {
  const [location] = useLocation();
  
  // Meta title and description based on route
  const getMetaData = () => {
    switch (location) {
      case '/':
        return {
          title: 'Fotello - AI-Powered Real Estate Photo Editing',
          description: 'Transform your real estate photos with Fotello\'s AI technology. Get professional quality editing with enhanced lighting, color correction, and object removal.',
        };
      case '/pricing':
        return {
          title: 'Pricing - Fotello | AI Photo Editing for Real Estate',
          description: 'View Fotello\'s flexible pricing plans for AI-powered real estate photo editing. Choose the plan that fits your real estate photography needs.',
        };
      case '/terms':
        return {
          title: 'Terms of Service - Fotello',
          description: 'Read Fotello\'s terms of service for using our AI-powered real estate photo editing platform.',
        };
      case '/privacy':
        return {
          title: 'Privacy Policy - Fotello',
          description: 'Learn about Fotello\'s privacy policy and how we handle your data when using our AI photo editing service.',
        };
      default:
        return {
          title: 'Fotello - AI-Powered Real Estate Photo Editing',
          description: 'Transform your real estate photos with Fotello\'s AI technology.',
        };
    }
  };
  
  const { title, description } = getMetaData();
  
  return (
    <>
      <Helmet>
        <title>{title}</title>
        <meta name="description" content={description} />
        <meta property="og:title" content={title} />
        <meta property="og:description" content={description} />
        <meta property="og:url" content={`https://fotello.co${location}`} />
        <link rel="canonical" href={`https://fotello.co${location}`} />
        
        {/* DNS Prefetching for external domains */}
        <link rel="dns-prefetch" href="https://fonts.googleapis.com" />
        <link rel="dns-prefetch" href="https://fonts.gstatic.com" />
        <link rel="dns-prefetch" href="https://app.fotello.co" />
        <link rel="dns-prefetch" href="https://cal.com" />
        
        {/* Preconnect to critical domains */}
        <link rel="preconnect" href="https://fonts.googleapis.com" crossOrigin="anonymous" />
        <link rel="preconnect" href="https://fonts.gstatic.com" crossOrigin="anonymous" />
        <link rel="preconnect" href="https://app.fotello.co" crossOrigin="anonymous" />
      </Helmet>
      
      <AnimatePresence mode="sync">
        {/* This component will scroll to top on route changes */}
        <ScrollToTop />
        <PageChangeMonitor />
        <Switch>
          <Route path="/" component={Home} />
          <Route path="/pricing" component={PricingPage} />
          <Route path="/terms" component={Terms} />
          <Route path="/privacy" component={Privacy} />
          <Route component={NotFound} />
        </Switch>
      </AnimatePresence>
    </>
  );
}

function App() {
  // Use effect to monitor initial page load performance
  useEffect(() => {
    // Start measuring after the component mounts
    const loadMeasurementId = 'app-load-complete';
    startMeasure(loadMeasurementId);
    
    // Initialize performance monitoring (runtime is already monitored separately)
    let cleanupRuntimePerformance = () => {};
    let cleanupWebVitalsMonitoring = () => {};
    
    // Only initialize in browser environments
    if (typeof window !== 'undefined') {
      try {
        // Dynamic import performance utilities to reduce initial bundle size
        import('./utils/runtime-performance').then(module => {
          cleanupRuntimePerformance = module.initRuntimePerformance();
        });
        
        // Core Web Vitals monitoring
        cleanupWebVitalsMonitoring = observeCoreWebVitals();
      } catch (error) {
        console.warn('Performance monitoring initialization failed:', error);
      }
    }
    
    // When page is fully loaded and rendered, complete the measurement
    const handleLoad = () => {
      // Wait for next frame to capture time after render
      requestAnimationFrame(() => {
        endMeasure(loadMeasurementId);
      });
    };
    
    if (document.readyState === 'complete') {
      handleLoad();
    } else {
      window.addEventListener('load', handleLoad);
    }
    
    // Return cleanup function
    return () => {
      window.removeEventListener('load', handleLoad);
      cleanupRuntimePerformance();
      cleanupWebVitalsMonitoring();
    };
  }, []);
  
  return (
    <ErrorBoundary>
      <QueryClientProvider client={queryClient}>
        <Router />
        <Toaster />
        <IntercomMessenger />
      </QueryClientProvider>
    </ErrorBoundary>
  );
}

export default App;
